from .build import Build
from dataclasses import dataclass, field
from SCons.Environment import Environment


@dataclass
class Target:
    name: str
    build: Build

    args: list[str] = field(default_factory=list)

    def __repr__(self) -> str:
        return f"{self.name} -> {self.build.name}"

    @property
    def action(self) -> str:
        return " ".join([self.build.target, *self.args])

    def register(self, env: Environment) -> None:
        alias = env.Alias(self.name, [self.build.target], self.action)
        env.AlwaysBuild(alias)



from .build import Build
from .compiler import flags
from .containers import flatten
from .environment import conan, packages
from .flag import Flag
from .routine import Routine
from .script import Script
from .target import Target
from .tasks import Tasks



from .containers import flatten
from dataclasses import dataclass, field
from SCons.Environment import Environment


@dataclass
class Script:
    name: str

    args: list[str | list[str]] = field(default_factory=list)
    prefix: list[str | list[str]] = field(default_factory=list)

    def __repr__(self) -> str:
        return self.name

    @property
    def action(self) -> str:
        return " ".join(flatten([*self.prefix, self.name, *self.args]))

    def register(self, env: Environment) -> None:
        alias = env.Alias(self.name, [], self.action)
        env.AlwaysBuild(alias)



import os
import psutil
from emoji import emojize
from SCons.Environment import Environment
from SCons.Script import SConscript


def conan(
    defines: list[str] | None = None,
    source: str = "SConscript_conandeps",
) -> tuple[Environment, list[str]]:
    if defines is None:
        defines = []

    env = Environment(
        num_jobs=psutil.cpu_count(),
        ENV={"PATH": os.getenv("PATH", "")},
        CXXCOMSTR=emojize(":wrench: Compiling $TARGET"),
        LINKCOMSTR=emojize(":link: Linking $TARGET"),
    )

    conandeps = SConscript(source)["conandeps"]
    conandeps["CPPDEFINES"] += defines

    env.MergeFlags(conandeps)
    return env


def packages(
    names: list[str],
    libs: list[str] | None = None,
    explicit: bool = False,
    source: str = "SConscript_conandeps",
) -> dict[str, list[str]]:
    if libs is None:
        libs = names.copy()

    if not explicit:
        names.append("conandeps")

    reduced = {"LIBS": libs}

    for name, package in SConscript(source).items():
        if name in names:
            for flag, exported in package.items():
                reduced[flag] = reduced.get(flag, []) + exported

    return reduced



from .script import Script
from dataclasses import dataclass, field
from SCons.Environment import Environment


@dataclass
class Routine:
    name: str
    scripts: list[Script] = field(default_factory=list)

    def __repr__(self) -> str:
        return f"{self.name} -> {self.scripts}"

    def register(self, env: Environment) -> None:
        alias = env.Alias(self.name, [script.name for script in self.scripts])
        env.AlwaysBuild(alias)



import os
from dataclasses import dataclass, field
from SCons.Environment import Environment


@dataclass
class Build:
    name: str

    files: list[str] = field(default_factory=list)
    flags: list[str] = field(default_factory=list)

    packages: dict[str, list[str]] = field(default_factory=dict)

    output: str = "dist"
    shared: bool = False

    def __repr__(self) -> str:
        return self.name

    @property
    def target(self) -> str:
        return os.path.join(self.output, self.name)

    @property
    def merge(self) -> dict[str, list[str]]:
        packages = self.packages.copy()
        packages["CXXFLAGS"] = packages.get("CXXFLAGS", []) + self.flags
        return packages

    def path(self, file: str) -> str:
        root = os.path.splitext(os.path.normpath(file))[0]
        return f"{root.replace('.', '-')}-[{self.name}]"

    def nodes(self, env: Environment) -> list[str]:
        return [env.Object(self.path(file), file, **self.merge) for file in self.files]

    def register(self, env: Environment) -> None:
        if self.shared:
            outputs = env.Library(self.target, self.nodes(env), **self.merge)
            env.Alias(self.name, outputs[0])
        else:
            env.Program(self.target, self.nodes(env), **self.merge)
            env.Alias(self.name, self.target)



from dataclasses import dataclass
from SCons.Environment import Environment
from SCons.Script.Main import AddOption


@dataclass
class Flag:
    name: str

    def __repr__(self) -> str:
        return self.name

    # pylint: disable=W0613
    def register(self, env: Environment) -> None:
        AddOption(self.name, action="store_true")



from .containers import flatten


def flags(
    standard: str, warnings: list[str] | None = None, ignore: list[str] | None = None
) -> list[str]:
    if warnings is None:
        warnings = []

    if ignore is None:
        ignore = []

    return flatten(
        [
            f"-std={standard}",
            [f"-W{warning}" for warning in warnings],
            [f"-Wno-{warning}" for warning in ignore],
        ]
    )



import sys
from .build import Build
from .flag import Flag
from .routine import Routine
from .script import Script
from .target import Target
from dataclasses import dataclass, field
from SCons.Environment import Environment


@dataclass
class Tasks:
    builds: list[Build] = field(default_factory=list)
    targets: list[Target] = field(default_factory=list)
    scripts: list[Script] = field(default_factory=list)
    routines: list[Routine] = field(default_factory=list)
    flags: list[Flag] = field(default_factory=list)

    @property
    def cli(self) -> str:
        sections = [
            ":".join([k, "".join([f"\n  {i}" for i in v] if len(v) > 0 else "\n  -")])
            for k, v in self.__dict__.items()
        ]

        return "".join(["\n", "\n\n".join(sections), "\n"])

    def dump(self) -> None:
        sys.stdout.write(f"{self.cli}\n")

    def register(self, env: Environment) -> None:
        for group in self.__dict__.values():
            for task in group:
                task.register(env)
